@@ -103,6 +103,7 @@ struct _GeditDocumentPrivate
 {
 	gchar	    *uri;
 	gint 	     untitled_number;
+	gchar       *short_name;
 
 	GFileInfo   *metadata_info;
 
@@ -371,6 +372,14 @@ gedit_document_set_property (GObject      *object,
 			gedit_document_set_newline_type (doc,
 							 g_value_get_enum (value));
 			break;
+		case PROP_SHORTNAME:
+			gedit_document_set_short_name_for_display (doc,
+			                                           g_value_get_string (value));
+			break;
+		case PROP_CONTENT_TYPE:
+			gedit_document_set_content_type (doc,
+			                                 g_value_get_string (value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -423,7 +432,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 	object_class->dispose = gedit_document_dispose;
 	object_class->finalize = gedit_document_finalize;
 	object_class->get_property = gedit_document_get_property;
-	object_class->set_property = gedit_document_set_property;	
+	object_class->set_property = gedit_document_set_property;
 
 	buf_class->mark_set = gedit_document_mark_set;
 	buf_class->changed = gedit_document_changed;
@@ -444,7 +453,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 							      "Short Name",
 							      "The document's short name",
 							      NULL,
-							      G_PARAM_READABLE |
+							      G_PARAM_READWRITE |
 							      G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (object_class, PROP_CONTENT_TYPE,
@@ -452,7 +461,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 							      "Content Type",
 							      "The document's Content Type",
 							      NULL,
-							      G_PARAM_READABLE |
+							      G_PARAM_READWRITE |
 							      G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (object_class, PROP_MIME_TYPE,
@@ -557,7 +566,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 
 
 	document_signals[LOADING] =
-   		g_signal_new ("loading",
+		g_signal_new ("loading",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GeditDocumentClass, loading),
@@ -731,6 +740,50 @@ get_default_style_scheme (void)
 	return def_style;
 }
 
+static void
+on_uri_changed (GeditDocument *doc,
+		GParamSpec    *pspec,
+		gpointer       useless)
+{
+#ifndef G_OS_WIN32
+	GFile *location;
+
+	location = gedit_document_get_location (doc);
+
+	/* load metadata for this uri: we load sync since metadata is
+	 * always local so it should be fast and we need the information
+	 * right after the uri was set.
+	 */
+	if (location != NULL)
+	{
+		GError *error = NULL;
+
+		if (doc->priv->metadata_info != NULL)
+			g_object_unref (doc->priv->metadata_info);
+
+		doc->priv->metadata_info = g_file_query_info (location,
+							      METADATA_QUERY,
+							      G_FILE_QUERY_INFO_NONE,
+							      NULL,
+							      &error);
+
+		if (error != NULL)
+		{
+			if (error->code != G_FILE_ERROR_ISDIR &&
+			    error->code != G_FILE_ERROR_NOTDIR &&
+			    error->code != G_FILE_ERROR_NOENT)
+			{
+				g_warning ("%s", error->message);
+			}
+
+			g_error_free (error);
+		}
+
+		g_object_unref (location);
+	}
+#endif
+}
+
 static GtkSourceLanguage *
 guess_language (GeditDocument *doc,
 		const gchar   *content_type)
@@ -756,22 +809,29 @@ guess_language (GeditDocument *doc,
 	else
 	{
 		GFile *file;
-		gchar *basename;
+		gchar *basename = NULL;
 
 		file = gedit_document_get_location (doc);
+		gedit_debug_message (DEBUG_DOCUMENT, "Sniffing Language");
 
 		if (file)
 		{
-			gedit_debug_message (DEBUG_DOCUMENT, "Sniffing Language");
-
 			basename = g_file_get_basename (file);
+		}
+		else if (doc->priv->short_name != NULL)
+		{
+			basename = g_strdup (doc->priv->short_name);
+		}
 
-			language = gtk_source_language_manager_guess_language (
-						gedit_get_language_manager (),
-						basename,
-						content_type);
+		language = gtk_source_language_manager_guess_language (
+					gedit_get_language_manager (),
+					basename,
+					content_type);
 
-			g_free (basename);
+		g_free (basename);
+
+		if (file != NULL)
+		{
 			g_object_unref (file);
 		}
 	}
@@ -797,6 +857,12 @@ on_content_type_changed (GeditDocument *doc,
 	}
 }
 
+static gchar *
+get_default_content_type (void)
+{
+	return g_content_type_from_mime_type ("text/plain");
+}
+
 static void
 gedit_document_init (GeditDocument *doc)
 {
@@ -811,7 +877,7 @@ gedit_document_init (GeditDocument *doc)
 
 	doc->priv->metadata_info = NULL;
 
-	doc->priv->content_type = g_content_type_from_mime_type ("text/plain");
+	doc->priv->content_type = get_default_content_type ();
 
 	doc->priv->readonly = FALSE;
 
@@ -859,6 +925,11 @@ gedit_document_init (GeditDocument *doc)
 			  "notify::content-type",
 			  G_CALLBACK (on_content_type_changed),
 			  NULL);
+
+	g_signal_connect (doc,
+			  "notify::uri",
+			  G_CALLBACK (on_uri_changed),
+			  NULL);
 }
 
 GeditDocument *
@@ -869,19 +940,33 @@ gedit_document_new (void)
 	return GEDIT_DOCUMENT (g_object_new (GEDIT_TYPE_DOCUMENT, NULL));
 }
 
+static void
+set_content_type_no_guess (GeditDocument *doc,
+			   const gchar   *content_type)
+{
+	gedit_debug (DEBUG_DOCUMENT);
+
+	if (doc->priv->content_type != NULL && content_type != NULL &&
+	    (0 == strcmp (doc->priv->content_type, content_type)))
+		return;
+
+	g_free (doc->priv->content_type);
+
+	if (content_type == NULL || g_content_type_is_unknown (content_type))
+		doc->priv->content_type = get_default_content_type ();
+	else
+		doc->priv->content_type = g_strdup (content_type);
+
+	g_object_notify (G_OBJECT (doc), "content-type");
+}
+
 static void
 set_content_type (GeditDocument *doc,
 		  const gchar   *content_type)
 {
 	gedit_debug (DEBUG_DOCUMENT);
 
-	g_free (doc->priv->content_type);
-
-	if (content_type != NULL)
-	{
-		doc->priv->content_type = g_strdup (content_type);
-	}
-	else
+	if (content_type == NULL)
 	{
 		GFile *file;
 		gchar *guessed_type = NULL;
@@ -899,43 +984,24 @@ set_content_type (GeditDocument *doc,
 			g_object_unref (file);
 		}
 
-		if (guessed_type == NULL || g_content_type_is_unknown (guessed_type))
-		{
-			g_free (guessed_type);
-			guessed_type = g_content_type_from_mime_type ("text/plain");
-		}
+		set_content_type_no_guess (doc, guessed_type);
 
-		doc->priv->content_type = guessed_type;
+		g_free (guessed_type);
 	}
-
-	g_object_notify (G_OBJECT (doc), "content-type");
-}
-
-#ifndef G_OS_WIN32
-static void
-query_info_cb (GFile         *source,
-	       GAsyncResult  *res,
-	       GeditDocument *doc)
-{
-	GError *error = NULL;
-
-	doc->priv->metadata_info = g_file_query_info_finish (source,
-							     res,
-							     &error);
-
-	if (error != NULL)
+	else
 	{
-		if (error->code != G_FILE_ERROR_ISDIR)
-			g_warning ("%s", error->message);
-		g_error_free (error);
-		
-		return;
+		set_content_type_no_guess (doc, content_type);
 	}
-
-	
-	on_content_type_changed (doc, NULL, NULL);
 }
-#endif
+
+void
+gedit_document_set_content_type (GeditDocument *doc,
+                                 const gchar   *content_type)
+{
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	set_content_type (doc, content_type);
+}
 
 static void
 set_uri (GeditDocument *doc,
@@ -960,27 +1026,12 @@ set_uri (GeditDocument *doc,
 		}
 	}
 
-#ifndef G_OS_WIN32
-	GFile *location;
-
-	/* Get the GFileInfo async so we can set the language from the metadata */
-	location = gedit_document_get_location (doc);
-	
-	if (location != NULL)
-	{
-		g_file_query_info_async (location,
-					 METADATA_QUERY,
-					 G_FILE_QUERY_INFO_NONE,
-					 G_PRIORITY_DEFAULT,
-					 NULL,
-					 (GAsyncReadyCallback) query_info_cb,
-					 doc);
-		g_object_unref (location);
-	}
-#endif
-
 	g_object_notify (G_OBJECT (doc), "uri");
-	g_object_notify (G_OBJECT (doc), "shortname");
+
+	if (doc->priv->short_name == NULL)
+	{
+		g_object_notify (G_OBJECT (doc), "shortname");
+	}
 }
 
 GFile *
@@ -1003,6 +1054,9 @@ void
 gedit_document_set_uri (GeditDocument *doc,
 			const gchar   *uri)
 {
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (uri != NULL);
+
 	set_uri (doc, uri);
 	set_content_type (doc, NULL);
 }
@@ -1026,13 +1080,27 @@ gedit_document_get_short_name_for_display (GeditDocument *doc)
 {
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), g_strdup (""));
 
-	if (doc->priv->uri == NULL)
+	if (doc->priv->short_name != NULL)
+		return g_strdup (doc->priv->short_name);
+	else if (doc->priv->uri == NULL)
 		return g_strdup_printf (_("Unsaved Document %d"),
 					doc->priv->untitled_number);
 	else
 		return gedit_utils_basename_for_display (doc->priv->uri);
 }
 
+void
+gedit_document_set_short_name_for_display (GeditDocument *doc,
+                                           const gchar   *short_name)
+{
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	g_free (doc->priv->short_name);
+	doc->priv->short_name = g_strdup (short_name);
+
+	g_object_notify (G_OBJECT (doc), "shortname");
+}
+
 gchar *
 gedit_document_get_content_type (GeditDocument *doc)
 {
@@ -1153,7 +1221,9 @@ document_loader_loaded (GeditDocumentLoader *loader,
 			GeditDocument       *doc)
 {
 	/* load was successful */
-	if (error == NULL || error->code == GEDIT_DOCUMENT_ERROR_CONVERSION_FALLBACK)
+	if (error == NULL ||
+	    (error->domain == GEDIT_DOCUMENT_ERROR &&
+	     error->code == GEDIT_DOCUMENT_ERROR_CONVERSION_FALLBACK))
 	{
 		GtkTextIter iter;
 		GFileInfo *info;
@@ -1187,7 +1257,6 @@ document_loader_loaded (GeditDocumentLoader *loader,
 			      gedit_document_loader_get_encoding (loader),
 			      (doc->priv->requested_encoding != NULL));
 
-
 		set_content_type (doc, content_type);
 
 		gedit_document_set_newline_type (doc,
