@@ -106,13 +106,11 @@ info_bar_add_stock_button_with_text (GtkInfoBar  *infobar,
 				     gint         response_id)
 {
 	GtkWidget *button;
-	
-	button = gedit_gtk_button_new_with_stock_icon (text, stock_id);
-	gtk_widget_show (button);
+	GtkWidget *image;
 
-	gtk_info_bar_add_action_widget (infobar,
-					button,
-					response_id);
+	button = gtk_info_bar_add_button (infobar, text, response_id);
+	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+	gtk_button_set_image (GTK_BUTTON (button), image);
 }
 #endif
 
@@ -221,6 +219,7 @@ parse_gio_error (gint          code,
 	switch (code)
 	{
 	case G_IO_ERROR_NOT_FOUND:
+	case G_IO_ERROR_NOT_DIRECTORY:
 		*error_message = g_strdup_printf (_("Could not find the file %s."),
 						  uri_for_display);
 		*message_details = g_strdup (_("Please check that you typed the "
@@ -368,8 +367,8 @@ parse_error (const GError *error,
 
 	if (error->domain == G_IO_ERROR)
 	{
-		ret = parse_gio_error (error->code, 
-				       error_message, 
+		ret = parse_gio_error (error->code,
+				       error_message,
 				       message_details,
 				       uri,
 				       uri_for_display);
@@ -377,7 +376,7 @@ parse_error (const GError *error,
 	else if (error->domain == GEDIT_DOCUMENT_ERROR)
 	{
 		ret = parse_gedit_error (error->code,
-					 error_message, 
+					 error_message,
 					 message_details,
 					 uri,
 					 uri_for_display);
@@ -432,8 +431,10 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 	}
 
 	if (error_message == NULL)
+	{
 		error_message = g_strdup_printf (_("Could not revert the file %s."),
 						 uri_for_display);
+	}
 
 	message_area = create_io_loading_error_message_area (error_message,
 							     message_details,
@@ -644,11 +645,9 @@ gedit_io_loading_error_message_area_new (const gchar         *uri,
 	         (error->domain == GEDIT_DOCUMENT_ERROR &&
 	         error->code == GEDIT_DOCUMENT_ERROR_ENCODING_AUTO_DETECTION_FAILED))
 	{
-		error_message = g_strdup_printf (_("Could not open the file %s."),
-						 uri_for_display);
 		message_details = g_strconcat (_("gedit has not been able to detect "
-				               "the character encoding."), "\n", 
-				               _("Please check that you are not trying to open a binary file."), "\n",
+					         "the character encoding."), "\n",
+					       _("Please check that you are not trying to open a binary file."), "\n",
 					       _("Select a character encoding from the menu and try again."), NULL);
 		convert_error = TRUE;
 	}
@@ -679,6 +678,12 @@ gedit_io_loading_error_message_area_new (const gchar         *uri,
 		parse_error (error, &error_message, &message_details, uri, uri_for_display);
 	}
 
+	if (error_message == NULL)
+	{
+		error_message = g_strdup_printf (_("Could not open the file %s."),
+						 uri_for_display);
+	}
+
 	if (convert_error)
 	{
 		message_area = create_conversion_error_message_area (error_message,
@@ -1193,8 +1198,10 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 	}
 
 	if (error_message == NULL)
+	{
 		error_message = g_strdup_printf (_("Could not save the file %s."),
 						 uri_for_display);
+	}
 
 	message_area = create_io_loading_error_message_area (error_message,
 							     message_details,
